import string
import types
import traceback
import tornado

class CatchExceptionMeta(type):
    def __new__(mcs, cls_name, bases, attrs):
        supported_methods = map(string.lower, 
            tornado.web.RequestHandler.SUPPORTED_METHODS)
        for attr in attrs:
            if attr in supported_methods and \
                isinstance(attrs[attr], types.FunctionType):
                attrs[attr] = CatchExceptionMeta.catch(attrs[attr])
        return type.__new__(mcs, cls_name, bases, attrs)

    @staticmethod
    def catch(f):
        def _inner(self, *a, **kw):
            try:
                return f(self, *a, **kw)
            except Exception as _:
                self.set_status(500)
                self.write(traceback.format_exc())
                self.finish()
        return _inner 




import re

class ChooseAPPMiddleware:
    def __init__(self, *apps):
        self._apps = apps

    def __call__(self, environment, start_response):
        d = {}
        for app in reversed(self._apps):
            d.update(dict.fromkeys(app.reflect_uri(), app))

        uri = environment['PATH_INFO']
        for uri_pattern in d:
            if re.match(uri_pattern, uri):
                return d[uri_pattern](environment, start_response)
        return self._apps[0](environment, start_response)

class DispatchMiddleWare:
    def __init__(self, app, path, argname, mapping, deny=True):
        self._app = app
        self._path = path
        self._argname = argname
        self._mapping = mapping
        self._deny = deny

    def __call__(self, environment, start_response):
        path = environment["PATH_INFO"]
        if self._deny and path in self._mapping.values():
            content = "page is not found"
            start_response("404 Not Found", [
                        ("Content-Length", str(len(content)))
                    ])
            return [content]
        if self._path == path:
            qr = environment["QUERY_STRING"]
            for one_arg in qr.split("&"):
                if not one_arg.startswith(self._argname+"="):
                    continue
                _, key = one_arg.split("=", 1)
                if key in self._mapping:
                    environment["PATH_INFO"] = self._mapping[key]
                break
        return self._app(environment, start_response)




import re
import keyword
import os
import tempfile
import logging
import pkgutil
import sys
import zipimport
import collections

VALIDATE_REGEXP = re.compile('^[_a-zA-Z][0-9a-zA-Z_]*$')

def __get_filename_maxlen():
    if 'win' in sys.platform:
        return 255
    with tempfile.NamedTemporaryFile() as tmpfile:
        return os.pathconf(tmpfile.name, 'PC_NAME_MAX')
FILENAME_MAXLEN = __get_filename_maxlen()

def is_valid_name(name, maxlen=FILENAME_MAXLEN):
    if (not isinstance(name, str)) or \
        keyword.iskeyword(name) or \
        (not VALIDATE_REGEXP.match(name)) or \
        len(name) > maxlen:
        return False
    return True

def makedirs(d):
    d = os.path.abspath(d)
    if os.path.exists(d):
        return "'%s' already exists" % d
    try:
        os.makedirs(d)
    except OSError as e:
        return str(e)

COLOR = {
    "red": "\033[31m",
    "green": "\033[32m",
}

def find_modules(*a, **kw):
    hookfunc = kw.pop('hookfunc', lambda *a, **kw: True)
    for module_loader, name, ispkg in pkgutil.iter_modules(*a, **kw):
        if name in sys.modules:
            yield sys.modules[name]
            continue
        if not hookfunc(name):
            continue
        try:
            if isinstance(module_loader, pkgutil.ImpImporter):
                yield module_loader.find_module(name).load_module(name)
            elif isinstance(module_loader, zipimport.zipimporter):
                yield module_loader.load_module(name)
            logging.debug("succeed to import module:%s", name)
        except Exception as exc:
            logging.warn("failed to import module:%s, because:%s",
                    name, str(exc))

def imports(module):
    if os.sep in module:
        _name = os.path.basename(module)
        _path = [os.path.dirname(module)]
    else:
        _name = module
        _path = None
    try:
        return next(find_modules(_path,
                hookfunc=lambda name: name==_name))
    except StopIteration:
        pass

def ignore_exceptions(*exc):
    def _inner(f):
        def _innest(*a, **kw):
            try:
                return f(*a, **kw)
            except tuple([_ for _ in exc if
                 issubclass(_, Exception)]) as e:
                return e
        return _innest
    return _inner

class Base(object):
    __slots__ = tuple()

    def __getattr__(self, attr):
        if attr not in self.__slots__:
            raise AttributeError("instance of %s has no attribute %s" %
                (self.__class__.__name__, attr))
        return object.__getattribute__(self, attr)

    def __setattr__(self, attr, value):
        if attr not in self.__slots__:
            raise AttributeError("not supported attribute %s" % attr)
        try:
            old_value = object.__getattribute__(self, attr)
        except AttributeError:
            return object.__setattr__(self, attr, value)
        value = isinstance(old_value, collections.MutableSequence) and \
            (old_value + [value]) or value
        return object.__setattr__(self, attr, value)

    def __delattr__(self, attr):
        raise RuntimeError("delete %s is not supported" % attr)

    def __getitem__(self, *a, **kw):
        return self.__getattr__(*a, **kw)

    def __setitem__(self, *a, **kw):
        return self.__setattr__(*a, **kw)

    def __delitem__(self, *a, **kw):
        return self.__delattr__(*a, **kw)

class Response(Base):
    __slots__ = ["headers", "status", "data", "will_render"]

    def __init__(self):
        self.headers = []
        self.status = 200
        self.data = None
        self.will_render = None

    @classmethod
    def initialize(cls, headers=[], status=200, data=None, will_render=None):
        instance = cls()
        for header in headers:
            instance.headers = header
        instance.status = status
        instance.data = data
        instance.will_render = will_render
        return instance




import os.path
import os
import shutil
import sys

from pkg_resources import resource_filename, resource_isdir, \
            resource_listdir, resource_string
from .util import is_valid_name, makedirs, COLOR

def show_and_exit(msg, exitcode=1, color=COLOR["red"]):
    print("%s%s\033[0m" % (color, msg))
    exit(exitcode)

def show_subcommands(exitcode=0, color=COLOR["green"]):
    subcommands = "\n".join([" "*4+attr[12:] for attr in 
        dir(_DispatchCommand) if attr.startswith('_subcommand_') and 
        callable(getattr(_DispatchCommand, attr))])
    show_and_exit(
"""Available subcommands:
[hybrid]
%s""" % subcommands, exitcode, color)

def copy_skeleton(base, to):
    if not resource_isdir(__name__, base):
        shutil.copy2(resource_filename(__name__, base), to)
        return
    for ele in resource_listdir(__name__, base):
        if not resource_isdir(__name__, base+'/'+ele):
            copy_skeleton(base+'/'+ele, to)
            continue
        os.makedirs(os.path.join(to, ele))
        copy_skeleton(base+'/'+ele, os.path.join(to, ele))

class _DispatchCommand:
    def __init__(self, args):
        if not args:
            show_subcommands()
        self._subcommand = args[0]
        self._args       = args[1:]

    def execute(self):
        subcommand_name = '_subcommand_' + self._subcommand
        if not hasattr(self, subcommand_name) or \
            not callable(getattr(self, subcommand_name)):
            show_and_exit(
"""Unknown command: '%s'
Type '%s help' for usage.""" % (self._subcommand, os.path.basename(__file__)))
        getattr(self, subcommand_name)()

    def _subcommand_help(self):
        show_subcommands()

    def _subcommand_startproject(self):
        if not self._args:
            show_and_exit("you must provide a project name")
        if not is_valid_name(self._args[0]):
            show_and_exit("'%s' is not a valid project name. "
"Please make sure the name begins with a letter or underscore." % self._args[0])
        basedir = os.path.join(self._args[0], self._args[0])
        result = makedirs(basedir)
        if result:
            show_and_exit(result)
        copy_skeleton('skeleton/project', basedir)
        with open(os.path.join(self._args[0], 'test_server.py'), 'wb') as fd:
            fd.write(resource_string(__name__, "skeleton/test_server.py.tmpl") % self._args[0])
        makedirs(os.path.join(self._args[0], 'tests/'))

    def _subcommand_linecount(self):
        if len(self._args) < 1 or not os.path.isdir(self._args[0]) or \
            not os.access(self._args[0], os.R_OK):
            show_and_exit("you must provide a directory with read permission")
        total_count = 0 
        for dir, subdirs, files in os.walk(self._args[0]):
            for file in files:
                file = os.path.join(dir, file)
                if os.path.splitext(file)[1] != ".py":
                    continue
                current_count = self.__get_line_count(file)
                print("%s %d" % (file, self, current_count))
                total_count = total_count + current_count
        print("total count %d" % total_count)

    def __get_line_count(self, filename):
        line_count = 0 
        with open(filename) as fd: 
            for line in fd: 
                if line.strip() and not line.strip().startswith("#"):
                    line_count = line_count + 1 
        return line_count

main = lambda args=None: _DispatchCommand(args or sys.argv[1:]).execute()




from ..settings import tornadoapp
from .base_handler import BaseHandler

@tornadoapp.route("/tornado")
class TornadoHandler(BaseHandler):
    def get(self):
        self.render_func({"uri": self.request.uri,
            "datetime": __import__("datetime").datetime.now()})




from flask import Flask

from .base_wsgi import BaseWSGI

class FlaskWSGI(Flask, BaseWSGI):
    def __init__(self, *a, **kw):
        Flask.__init__(self, *a, **kw)
        BaseWSGI.__init__(self)

    def reflect_uri(self):
        #TODO: convert urlrule to regexp!
        return [rule.rule for rule in self.url_map.iter_rules()]




import types
import re

from DBUtils import PooledDB

class InterfaceError(StandardError):
    pass

class db_pool(object):
    def __init__(self, db_api, **kwargs):
        if type(db_api) == types.ModuleType:
            self._db_api = db_api
        elif isinstance(db_api, basestring):
            self._db_api = __import__(db_api)
        else:
            raise InterfaceError(
                'db_api should be a module '
                'or basestring object, not %s.' % type(db_api).__name__)

        self._kwargs = kwargs
        self.mincached = self._kwargs.pop('mincached', 1)
        self.maxcached = self._kwargs.pop('maxcached', 10)
        self.maxshared = self._kwargs.pop('maxshared', 5)
        self.maxconnections = self._kwargs.pop('maxconnections', 0)
        self.blocking = self._kwargs.pop('blocking', True)
        self.ping = self._kwargs.pop('ping', 0)

        self._pool = None

    def create_pool(self):
        if self._pool is None:
            self._pool = PooledDB.PooledDB(self._db_api,
                                            self.mincached,
                                            self.maxcached,
                                            self.maxshared,
                                            self.maxconnections,
                                            self.blocking,
                                            self.ping,
                                            **self._kwargs)

    def get_conn(self):
        if self._pool is None:
            self.create_pool()
        return self._pool.connection()

    def close(self):
        if hasattr(self._pool, 'close') and callable(self._pool.close):
            self._pool.close()
        self._pool = None

    def __del__(self):
        self.close()

class db(object):
    def __init__(self, pool):
        assert isinstance(pool, db_pool)
        self._pool = pool

    def __del__(self):
        self._pool.close()

    def _create(self):
        conn = self._pool.get_conn()
        cursor = conn.cursor()
        return conn, cursor

    def _close(self, conn=None, cursor=None):
        if cursor and hasattr(cursor, 'close') and callable(cursor.close):
            cursor.close()
        if conn and hasattr(conn, 'close') and callable(conn.close):
            conn.close()

    def _commit(self, conn):
        if hasattr(conn, 'commit') and callable(conn.commit):
            conn.commit()

    def query(self, table, fields, where=' 1 = 1'):
        if not isinstance(fields, (list, tuple)):
            fields = [fields]
        sql = 'SELECT %s FROM %s WHERE %s' % (','.join(fields), table, where)

        conn, cursor = self._create() 
        cursor.execute(sql)
        for row in cursor:
            yield dict(zip(fields, row))
        self._close(conn, cursor)

    def execute(self, sql, args=None, commit=True, lastrowid=False):
        try:
            conn, cursor = self._create()
            result = cursor.execute(sql, args)
            if commit:
                self._commit(conn)
            return result if not lastrowid else cursor.lastrowid
        finally:
            self._close(conn,cursor)

    def update(self, table, row, where, escape=True):
        arr = []
        for col, val in row.iteritems():
            if isinstance(val, basestring):
                if escape: 
                    val = self.escape_string(val)
                arr.append("%s = '%s'" % (col, val))
            else:
                arr.append("%s = %s" % (col, val if val is not None else 'null'))
        sql = "UPDATE %s SET %s WHERE %s" % (table, ' , '.join(arr), where)
        return self.execute(sql)

    def escape_string(self, s):
        ESCAPE_REGEX = re.compile(r"[\0\n\r\032\'\"\\]")
        ESCAPE_MAP = {'\0':'\\0', '\n':'\\n', '\r':'\\r', '\032':'\\Z',
                        '\'':'\\\'', '"':'\\"', '\\':'\\\\'}
        return ("%s" % ESCAPE_REGEX.sub(
                lambda match: ESCAPE_MAP.get(match.group(0)), s))

    def query_one(self, *args, **kwargs):
        rs = self.query(*args, **kwargs)
        if not rs: return rs
        for r in rs:
            return r

    def insert(self, table, fields, escape=True, update=''):
        items = map(lambda t:(t[0], (self.escape_string(t[1])
            if isinstance(t[1], basestring) and escape else t[1])), fields.items())
        sql = 'INSERT INTO %s SET ' % table + ','.join(map(lambda t:'%s=%s' % (t[0], 
                'null' if t[1] is None else "'%s'"%t[1]), items)) + ' ' + update
        return self.execute(sql, lastrowid=True)




import json
import datetime

def __json_default(obj):
    if isinstance(obj, datetime.datetime):
        return obj.strftime('%F %T')
    elif isinstance(obj, datetime.date):
        return obj.strftime('%F')
    elif isinstance(obj, datetime.time):
        return obj.strftime('%T')
    raise TypeError("%r is not JSON serializable" % obj)

def json_view(data, *a, **kw):
    return json.dumps(data, 
            default=kw.pop('default', __json_default), 
            *a, **kw)

if __name__ == "__main__":
    print json_view({'datetime': datetime.datetime.now(),
                        'date': datetime.date.today()}, indent=2)




import urllib
import tempfile
import urlparse

class TestWSGIApplication:
    HTTP_METHODS = ('GET', 'HEAD', 'POST', 'DELETE', 'PATCH', 'PUT', 'OPTIONS')
    def __init__(self, app, server_name=None):
        self._app = app
        self._chunks = [] #storing data written by ``write``
        self._env = {} #storing WSGI environment variables
        self._env['SERVER_NAME'] = server_name or ''
        self.initialize()

    def initialize(self):
        self._env['SERVER_PROTOCOL'] = 'HTTP/1.1'
        self._env['wsgi.version'] = (1, 0)
        self._env['wsgi.input'] = tempfile.NamedTemporaryFile()
        self._env['wsgi.errors'] = tempfile.NamedTemporaryFile()
        self._env['wsgi.multithread'] = False
        self._env['wsgi.multiprocess'] = False
        self._env['wsgi.run_once'] = False

    def start_response(self, status, headers, exc_info=None):
        if not headers:
            raise RuntimeError('MissingResponseHeaders')
        self._status = status
        self.status_code = int(status.split()[0])
        self.headers = dict(headers)
        self.exc_info = exc_info
        return self.write

    def write(self, chunk):
        self._chunks.append(chunk)

    def _wrap(self, url, headers={}, params={}, data=''):
        url = urlparse.urlparse(url)
        if not url.scheme:
            raise RuntimeError('MissingSchema')
        self._env['wsgi.url_scheme'] = url.scheme
        self._env['PATH_INFO'] = url.path
        self._env['QUERY_STRING'] = urllib.urlencode(params)
        if not self._env['QUERY_STRING']:
            self._env['QUERY_STRING'] = url.query
        else:
            self._env['QUERY_STRING'] += ('' if not url.query else ('&'+url.query))
        self._env['wsgi.input'].write(data)
        self._env['wsgi.input'].seek(0)
        self._env['CONTENT_LENGTH'] = len(data)

        if headers:
            for k, v in headers.items():
                self._env['HTTP_'+k.replace('-', '_').upper()] = v
        self.response = self.finalize()
        return self

    def __str__(self):
        return self.response

    def __getattr__(self, attr):
        if not attr.upper() in self.HTTP_METHODS:
            raise AttributeError('instance of %r has no attribute: %s.'
                    % (self.__class__, attr))
        self._env['REQUEST_METHOD'] = attr.upper()
        return self._wrap

    def finalize(self):
        gen = list(self._app(self._env, self.start_response))
        self.body = ''.join(self._chunks) + ''.join(gen)

        return \
            '%s '%self._env['SERVER_PROTOCOL'] + self._status + '\r\n' + \
            "\r\n".join(['%s: %s' % (k, v) for k, v in self.headers.items()]) + \
            "\r\n\r\n" + \
            self.body




import types
import collections
from google.protobuf.internal.containers \
    import RepeatedScalarFieldContainer, RepeatedCompositeFieldContainer

cpp_implementation = True
try:
    import google.protobuf.pyext._message as __message
except ImportError:
    cpp_implementation = False

_simple_types = (int, long, float, basestring, bool)
_list_types   = (RepeatedScalarFieldContainer, 
                    RepeatedCompositeFieldContainer)
if cpp_implementation:
    _list_types = _list_types + (__message.RepeatedCompositeContainer,
                    __message.RepeatedScalarContainer)

def pb2json(pb, strip_keys=None):
    result = {}
    def get_json(pb, result):
        for field_descriptor, value in pb.ListFields():
            if strip_keys and field_descriptor.name in strip_keys:
                continue
            if isinstance(value, _simple_types):
                result[field_descriptor.name] = value
            elif isinstance(value, _list_types):
                for one_value in value._values if hasattr(value, "_values") else value:
                    if isinstance(one_value, _simple_types):
                        result.setdefault(field_descriptor.name, []). \
                            append(one_value)
                    else:
                        result.setdefault(field_descriptor.name, []). \
                            append(get_json(one_value, {}))
            else:
                get_json(value, result.setdefault(field_descriptor.name, {}))
        return result
    return get_json(pb, result)

class json2pb:
    def __init__(self, pb_module, root_name, strip_keys=None):
        self._module = pb_module if type(pb_module) == types.ModuleType \
                            else __import__(pb_module)
        self._root = getattr(self._module, root_name)()
        self._strip_keys = strip_keys or []

    def deal_sequence(self, k, v, root):
        for _ in v:
            if isinstance(_, _simple_types):
                getattr(root, k).append(_)
            elif isinstance(_, collections.Sequence):
                pass
            elif isinstance(_, collections.Mapping):
                node = getattr(root, k).add()
                self.deal_mapping(_, node)
        return root

    def deal_mapping(self, obj, root):
        for k, v in obj.items():
            if k in self._strip_keys:
                continue
            if isinstance(v, _simple_types):
                setattr(root, k, v)
            elif isinstance(v, collections.Mapping):
                self.deal_mapping(v, getattr(root, k))
            elif isinstance(v, collections.Sequence):
                self.deal_sequence(k, v, root)
        return root

    def get_pb(self, json_obj):
        assert isinstance(json_obj, collections.Mapping)
        self.deal_mapping(json_obj, self._root)
        return self._root




#import os
from hybrid.wsgi_app import tornadoapp
from hybrid.util import imports

#curdir = os.path.dirname(os.path.abspath(__file__))
apps = [tornadoapp]
controllers = [
    #os.path.join(curdir, 'controllers'),
]

def install_controllers():
    #to avoid circular reference
    for controller in controllers:
        imports(controller)
    from . import controllers as _




from setuptools import setup, find_packages

setup(
    name='hybrid',
    version='1.2.0',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires = [
        "tornado>=3.1.1",
        "flask",
        "DBUtils",
    ],
    entry_points={
        "console_scripts": [
            "hybrid = hybrid.commands:main",
        ],
    },

    author='timchow',
    author_email='jordan23nbastar@yeah.net',
    url='http://timd.cn/',
    description='hybrid tools set',
    keywords='hybrid server',
    license='LGPL',
)



from tornado.web import RequestHandler
from tornado.wsgi import WSGIApplication

from .base_wsgi import BaseWSGI

class TornadoWSGI(BaseWSGI):
    def __init__(self, *a, **kw):
        BaseWSGI.__init__(self)
        self._wsgi_app = WSGIApplication(*a, **kw)

    def __getattr__(self, attr):
        if hasattr(self._wsgi_app, attr):
            return getattr(self._wsgi_app, attr)
        raise AttributeError("instance of `%s' has no attribute: %s" % 
                (self.__class__.__name__, attr))

    def route(self, uri, host_pattern='.*$'):
        if not isinstance(uri, str):
            raise TypeError("expected string or buffer")
        def _inner(handler_class):
            if not issubclass(handler_class, RequestHandler):
                raise RuntimeError('expected subclass of RequestHandler')
            self._wsgi_app.add_handlers(host_pattern, 
                [(uri, handler_class)])
            handler_class.__register_app__ = self
            return handler_class
        return _inner

    def reflect_uri(self):
        uris = []
        for _, urlspecs in self._wsgi_app.handlers:
            for urlspec in urlspecs:
                uris.append(urlspec.regex.pattern)
        return uris




import unittest

import pb_test_pb2
import simple_convert

class TestPb2Json(unittest.TestCase):
    def setUp(self):
        self.pbobj = pb_test_pb2.m1()
        self.pbobj.int1 = 1
        self.pbobj.arr1.extend([11, 33, 55, 77])

        a2_1 = self.pbobj.arr2.add()
        a2_1.int2 = 2
        a3_1 = a2_1.arr3.add()
        a3_1.int3 = 88
        a3_2 = a2_1.arr3.add()
        a3_2.int3 = 99

        a2_2 = self.pbobj.arr2.add()
        a2_2.int2 = 3
        a3_1 = a2_2.arr3.add()
        a3_1.int3 = 1000
        a3_2 = a2_2.arr3.add()
        a3_2.int3 = 2000

        self.jsonobj = {
            "int1": 1,
            "arr1": [11, 33, 55, 77],
            "arr2": [{"int2": 2, "arr3": [{"int3": 88}, {"int3": 99}]},
                        {"int2": 3, "arr3": [{"int3": 1000}, {"int3": 2000}]}]}

    def runTest(self):
        self.assertEqual(simple_convert.pb2json(self.pbobj), self.jsonobj)

class TestJson2pb(unittest.TestCase):
    def setUp(self):
        self.jsonobj = {
            "int1": 1,
            "arr1": [11, 33, 55, 77],
            "arr2": [{"int2": 2, "arr3": [{"int3": 88}, {"int3": 99}]},
                        {"int2": 3, "arr3": [{"int3": 1000}, {"int3": 2000}]}]}

    def runTest(self):
        pb = simple_convert.json2pb("pb_test_pb2", "m1").get_pb(self.jsonobj)
        pbstr = pb.SerializeToString()
        m = pb_test_pb2.m1()
        m.ParseFromString(pbstr)

        self.assertEqual(m.int1, self.jsonobj["int1"])
        self.assertEqual(m.arr1._values, self.jsonobj["arr1"])
        for ind, item in enumerate(m.arr2._values):
            self.assertEqual(item.int2, self.jsonobj["arr2"][ind]["int2"])
            for ind2, item2 in enumerate(item.arr3._values):
                self.assertEqual(item2.int3, 
                    self.jsonobj["arr2"][ind]["arr3"][ind2]["int3"])

if __name__ == "__main__":
    unittest.main()




from collections import Sequence

from tornado.web import RequestHandler
from hybrid.util import imports
from hybrid.metaclass import CatchExceptionMeta

class CastException(Exception):
    pass

class BaseHandler(RequestHandler):
    __metaclass__ = CatchExceptionMeta
    def getviewfunc(self, view, module):
        if not view and not module:
            from ..views import json_view
            return json_view
        elif not view:
            raise RuntimeError("missing view function name")

        if not module:
            from .. import views
            m = views
        else:
            m = imports(module)
        if not m or not hasattr(m, view):
            raise RuntimeError('can\'t find %s:%s' % (module, view))
        return getattr(m, view)

    def render_func(self, data, view=None, module=None, *a, **kw):
        self.write(self.getviewfunc(view, module)(data, *a, **kw))
        self.finish()

    def cast(self, v, t): 
        try:
            return t(v)
        except:
            return CastException()

    def param_check(self, args, howto=None):
        howto = howto or (lambda a: self.get_argument(a, None))
        for arg in args:
            arg = list(arg)
            if len(arg) == 1:
                arg += [(type, )]
            if len(arg) == 2:
                arg += [lambda *a, **kw: True]
            if not isinstance(arg[1], Sequence):
                arg[1] = (arg[1], )
            else:
                arg[1] = tuple(arg[1])

            value = howto(arg[0])
            if value is None and None in arg[1]:
                continue
            elif value is None and None not in arg[1]:
                return False

            def __check(t):
                value2 = self.cast(value, t)
                if not isinstance(value2, CastException) and \
                    isinstance(value2, arg[1]) and \
                    arg[2](value2):
                    return True
            if not any([__check(t) for t in arg[1]]):
                return False
        return True

